;;;; entropy-port.scm
;;;; Kon Lovett, Oct '09

(module entropy-port

  (;export
    ;
    entropy-port-lifetime
    ;
    make-entropy-source/port
    make-entropy-source/port-open
    make-entropy-source/port-open-timed
    make-entropy-source/file)

  (import
    scheme
    chicken
    (only type-checks
      check-input-port check-procedure check-symbol check-string)
    (only type-errors warning-argument-type)
    entropy-source
    entropy-support
    timed-resource)

  (require-library
    type-checks type-errors
    entropy-source entropy-support
    timed-resource)

;;

(define ((make-open-binary-input-file namstr)) (open-input-file namstr #:binary))

;;

#; ;NOT YET
(define (*make-entropy-source/timed-resource tr-ctor u8gen f64gen name docu)
  (let ((tr (tr-ctor)))
    (*make-entropy-source
      ;
      (lambda () (*make-entropy-source/timed-resource tr-ctor u8gen f64gen name docu))
      ;
      name
      ;
      docu
      ;
      (lambda () (with-timed-resource tr u8gen))
      ;
      (lambda () (with-timed-resource tr f64gen))
      ;
      (lambda (u8cnt u8vec)
        (with-timed-resource tr (lambda (r) (entropic-u8vector-filled u8cnt u8vec (u8gen r)))))
      ;
      (lambda (f64cnt f64vec)
        (with-timed-resource tr (lambda (r) (entropic-f64vector-filled f64cnt f64vec (f64gen r))))) ) ) )

(define (*make-entropy-source/timed-port-resource tr-ctor name docu)
  (let ((tr (tr-ctor)))
    (*make-entropy-source
      ;
      (lambda () (*make-entropy-source/timed-port-resource tr-ctor name docu))
      ;
      name
      ;
      docu
      ;
      (lambda () (with-timed-resource tr port-entropic-u8))
      ;
      (lambda () (with-timed-resource tr port-entropic-f64))
      ;
      (lambda (u8cnt u8vec)
        (with-timed-resource tr (cut port-entropic-u8vector <> u8cnt u8vec)))
      ;
      (lambda (f64cnt f64vec)
        (with-timed-resource tr (cut port-entropic-f64vector <> f64cnt f64vec))) ) ) )

(define (*make-entropy-source/port-open-timed opener timeout name docu)
  (*make-entropy-source/timed-port-resource
    (lambda () (make-timed-resource opener close-input-port timeout name))
    name docu) )

(define (*make-entropy-source/port port name docu)
  (*make-entropy-source
    ;
    (lambda () (*make-entropy-source/port port name docu))
    ;
    name
    ;
    docu
    ;
    (lambda () (port-entropic-u8 port))
    ;
    (lambda () (port-entropic-f64 port))
    ;
    (lambda (u8cnt u8vec) (port-entropic-u8vector port u8cnt u8vec))
    ;
    (lambda (f64cnt f64vec) (port-entropic-f64vector port f64cnt f64vec))) )

(define (*make-entropy-source/port-open opener name docu)
  (let ((to (entropy-port-lifetime)))
    (if to
        ;then auto-close on timeout
        (*make-entropy-source/port-open-timed opener to name docu)
        ;else keep open
        (let ((port (opener)))
          (set-finalizer! port close-input-port)
          (*make-entropy-source/port port name docu) ) ) ) )

;;; Timeout Seconds

(define-constant DEFAULT-ENTROPY-PORT-CLOSE-SECONDS 60.0)

(define entropy-port-lifetime
  (let ((lt DEFAULT-ENTROPY-PORT-CLOSE-SECONDS))
    (lambda args
      (if (null? args) lt
          (let ((x (car args)))
            (cond
              ((not x)                        (set! lt #f) )
              ((and (real? x) (positive? x))  (set! lt x) )
              (else
                (warning-argument-type 'entropy-port-lifetime x 'seconds) ) ) ) ) ) ) )

;;; Entropy from some port

(define (make-entropy-source/port port
          #!optional
            (name (gensym 'port-))
            (docu "Entropy from an open port"))
  (check-input-port 'make-entropy-source/port port)
  (check-symbol 'make-entropy-source/port name 'name)
  (check-string 'make-entropy-source/port docu 'documentation)
  (*make-entropy-source/port port name docu) )

;;; Entropy from port, timed or fixed (parameterized by entropy-port-lifetime)

(define (make-entropy-source/port-open opener
          #!optional
            (name (gensym 'port-))
            (docu "Entropy from port"))
  (check-procedure 'make-entropy-source/port-open opener 'open-procedure)
  (check-symbol 'make-entropy-source/port-open name 'name)
  (check-string 'make-entropy-source/port-open docu 'documentation)
  (*make-entropy-source/port-open opener name docu) )

;;; Make TImed Port Entropy Source

(define (make-entropy-source/port-open-timed opener timeout
          #!optional
            (name (gensym 'timed-port-))
            (docu "Entropy from timed open port"))
  (check-procedure 'make-entropy-source/port-open-timed opener 'open-procedure)
  (check-symbol 'make-entropy-source/port-open-timed name 'name)
  (check-string 'make-entropy-source/port-open-timed docu 'documentation)
  (*make-entropy-source/port-open-timed opener timeout name docu) )

;;; Entropy from some file (binary)

(define (make-entropy-source/file namstr
          #!optional
            (name (gensym 'file-))
            (docu (string-append "Entropy from file \"" namstr "\"")))
  (check-string 'make-entropy-source/file namstr 'filename)
  (check-symbol 'make-entropy-source/file name 'name)
  (check-string 'make-entropy-source/file docu 'documentation)
  (*make-entropy-source/port-open (make-open-binary-input-file namstr) name docu) )

) ;module entropy-port
